;;;
;;; Syntax-based highlights
;;;

;;; Comments, metas, and pub
(block_comment) @comment
(line_comment) @comment
(doc_comment) @comment.documentation
(inner_doc_comment) @comment.documentation
(file_meta) @attribute
(meta) @attribute
"pub" @keyword

;;; Imports
(import
  "import" @keyword
  ";" @punctuation.delimiter)
(import_tree_leaf_identifier
  (identifier) @type
  ; (#match? @type "^\\p{Lu}") ; TODO: Neovim doesn't support this regex...
  (#match? @type "^[A-Z]"))
(import_tree_leaf_identifier
  (identifier) @variable
  ; (#match? @variable "^[^\\p{Lu}]") ; TODO: Neovim doesn't support this regex...
  (#match? @variable "^[^A-Z]"))
(import_tree_leaf_operator
  "_" @operator
  (operator) @operator)
(import_tree_branch
  (identifier) @module
  "::" @punctuation.delimiter)
(import_tree_wild) @constant.builtin

;;; Declarations
(value_definition
  "def" @keyword
  name: (identifier) @variable
  ":" @punctuation.delimiter)

(function_definition
  "def" @keyword
  name: (identifier) @function)

(type_definition
  "type" @keyword
  name: (identifier) @type
  ":=" @punctuation.delimiter
  ";" @punctuation.delimiter)
(type_definition ":" @punctuation.delimiter)

(newtype_definition
  "newtype" @keyword
  name: (identifier) @type
  body_name: (identifier) @property
  ":" @punctuation.delimiter)
(newtype_definition ":" @punctuation.delimiter)

(record_definition
  "record" @keyword
  name: (identifier) @type)
(record_field
  name: (identifier) @property
  ":" @punctuation.delimiter)

(variant_definition
  "variant" @keyword
  name: (identifier) @type)
(variant_constructor
  name: (identifier) @constructor)
(variant_constructor
  ":" @punctuation.delimiter)

(kind_definition
  "kind" @keyword
  name: (identifier) @type)
(kind_constructor
  name: (identifier) @constructor)
(kind_constructor
  ":" @punctuation.delimiter)

(language_definition
  "lang" @keyword
  name: (identifier) @type)
(language_definition
  "extends" @keyword)
(lang_clause
  nt: (qualified_name) @type
  "." @punctuation.delimiter
  kwd: ["annots" "attrs" "prods"] @keyword
  op: [":=" "+=" "-="] @operator)
(lang_clause
  kwd: ["annots" "attrs"]
  (lang_clause_item
    name: (qualified_name
            name: (identifier) @property)))
(lang_clause
  kwd: "prods"
  (lang_clause_item
    name: (qualified_name
            name: (identifier) @constructor)))

(nt_definition
  "nt" @keyword
  name: (identifier) @type
  ";" @punctuation.delimiter)

(production_definition
  "prod" @keyword
  name: (identifier) @constructor
  ":" @punctuation.delimiter)
(production_definition ";" @punctuation.delimiter)

(aspect_definition
  "aspect" @keyword
  name: (qualified_name
          name: (identifier) @constructor))
(aspect_definition ":" @punctuation.delimiter)

(attribute_or_annotation_definition
  (attribute_or_annotation_keyword) @keyword
  name: (identifier) @function.method
  ":" @punctuation.delimiter
  ";" @punctuation.delimiter)

(typeclass_definition
  "class" @keyword
  name: (identifier) @type)
(class_value
  "def" @keyword
  name: (identifier) @variable
  ":" @punctuation.delimiter
  ";" @punctuation.delimiter)
(class_function
  "def" @keyword
  name: (identifier) @function
  ";" @punctuation.delimiter)
(class_type
  "type" @keyword
  name: (identifier) @type
  ";" @punctuation.delimiter)
(class_type ":" @punctuation.delimiter)

(typeclass_implementation
  "impl" @keyword)
(impl_type
  "type" @keyword
  name: (identifier) @type
  ":=" @punctuation.delimiter
  ";" @punctuation.delimiter)
(impl_type ":" @punctuation.delimiter)

(operator_definition
  "infix" @keyword
  name: (operator) @operator
  associativity: (_) @keyword
  ":=" @punctuation.delimiter
  body: (qualified_name
          name: (identifier) @function)
  ";" @punctuation.delimiter)

(submodule_declaration
  "mod" @keyword
  name: (identifier) @module
  ";" @punctuation.delimiter)

;;; Expressions
;; This needs to be first, because we override highlighting for variables (e.g.
;; in the function position of a call) later on.
(expr_variable
  (qualified_name
    name: (identifier) @variable))
;; The rest of these shouldn't overlap, so don't need an ordering relationship.
(expr_fn "fn" @keyword)
(expr_fn "=>" @punctuation.delimiter)
(expr_fn_term_binder ":" @punctuation.delimiter)
(expr_annot ":" @punctuation.delimiter)
(expr_operator
  (qualified_operator
    name: (operator) @operator))
(expr_attr
  "#" @punctuation.delimiter
  attr: (qualified_name
          name: (identifier) @property))
(expr_call
  func: (expr_variable
          (qualified_name
            name: (identifier) @constructor
            ;; TODO: Neovim doesn't support this regex...
            ;; (#match? @constructor "^\\p{Lu}")
            (#match? @constructor "^[A-Z]"))))
(expr_call
  func: (expr_variable
          (qualified_name
            name: (identifier) @function
            ;; TODO: Neovim doesn't support this regex...
            ;; (#match? @function "^[^\\p{Lu}]")
            (#match? @function "^[^A-Z]"))))
(expr_conditional
  "."  @punctuation.delimiter
  "if" @keyword)
(expr_conditional "else" @keyword)
(expr_dot_todo "?" @variable.builtin)
(expr_field
  "." @punctuation.delimiter
  field: (identifier) @property)
(expr_match
  "." @punctuation.delimiter
  "match" @keyword)
(match_case
  "=>" @punctuation.delimiter)
; projection expression
(expr_tyapp
  (expr_variable
    (qualified_name
      name: (identifier) @constructor
      ;; TODO: Neovim doesn't support this regex...
      ;; (#match? @constructor "^\\p{Lu}")
      (#match? @constructor "^[A-Z]"))))
(expr_tyapp
  (expr_variable
    (qualified_name
      name: (identifier) @function
      ;; TODO: Neovim doesn't support this regex...
      ;; (#match? @function "^[^\\p{Lu}]")
      (#match? @function "^[^A-Z]"))))
; block expression
(expr_placeholder "_" @variable.builtin)
(expr_record
  name: (qualified_name
    name: (identifier) @constructor))
(expr_record_field ":=" @punctuation.delimiter)
(expr_this) @variable.builtin
(expr_todo "?" @variable.builtin)

;;; Statements
(stmt_value_definition
  "def" @keyword
  name: (identifier) @variable)
(stmt_value_definition ":" @punctuation.delimiter)
(stmt_function_definition
  "def" @keyword
  name: (identifier) @function)
(stmt_expr ";" @punctuation.delimiter)

;;; Patterns
(pattern_alt
  "|" @punctuation.delimiter)
(pattern_view
  "<-" @punctuation.delimiter)
(pattern_alias
  "@" @punctuation.delimiter
  (identifier) @variable)
(pattern_record
  name: (qualified_name
    name: (identifier) @constructor))
(record_field_pattern
  name: (identifier) @property)
(record_field_pattern
  ":=" @punctuation.delimiter)
(pattern_variant
  (qualified_name
    name: (identifier) @constructor))
(pattern_var (identifier) @variable)
(pattern_wildcard (wildcard) @variable)

;;; Types
(type_type) @type.builtin
(type_fn "fn" @keyword)
(type_forall
  "forall" @keyword
  "->" @punctuation.delimiter)
(type_dot
  "." @punctuation.delimiter
  name: (identifier) @type)
(type_eq "=" @type)
(type_infer "?" @type)
(type_lang) @type.builtin
(type_nt) @type.builtin
(type_placeholder "_" @type)
(type_variable
  (qualified_name
    name: (identifier) @type))

(effect_row "..." @punctuation.delimiter)

;;; Utilities
(definition_body
  ":=" @punctuation.delimiter
  ";"  @punctuation.delimiter)
(definition_term_binder ":" @punctuation.delimiter)
(fn_arrow
  "->" @punctuation.delimiter)
(qualified_name "::" @punctuation.delimiter)
(qualified_name module: (identifier) @module)
(qualified_operator "::" @punctuation.delimiter)
(qualified_operator module: (identifier) @module)
(type_binder name: (identifier) @type)
(type_binder name: (wildcard) @type)
(type_binder ":" @punctuation.delimiter)
(where_clause
  "where" @keyword)

;;;
;;; Token-based highlights
;;;

;;; Brackets
["(" ")" "[" "]" "{" "}"] @punctuation.bracket
["-{" "}>"] @punctuation.bracket

;;; Literals
(byte_literal) @string
(bytes_literal) @string
(floating_point_literal) @number
(integer_literal) @number
(rune_literal) @string
(string_literal) @string
"false" @constant.builtin
"true" @constant.builtin

;;; Punctuation
"," @punctuation.delimiter
